using System;
using System.Collections;
using System.Collections.Generic;
using System.Xml.Linq;
using System.Linq;
using System.Net;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

using Microsoft.Lync.Model.Room;
using Microsoft.Lync.Model;
using System.Collections.ObjectModel;
using System.ComponentModel;



namespace LyncPersistentChat_AddIn
{
    public partial class Page : UserControl
    {
        List<PersistentChatDay> _chatRoomData;

        //TODO: 4.2.1 - Declare Room instance
        private readonly Room _room = null;

        public Page()
        {
            InitializeComponent();
            try
            {
                //TODO: 4.2.2 - Get the hosting room instance
                //Get the hosting chat room
                _room = (Room)LyncClient.GetHostingRoom();


                //TODO: 4.2.3 - Set up MessageReceived handler and retrieve latest messages
                if (_room != null)
                {
                    _room.BeginRetrieveLatestMessages(1000, GetMessagesCallback, "Latest");
                    _room.MessagesReceived += new EventHandler<RoomMessagesEventArgs>(_room_MessagesReceived);
                }


            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        /// <summary>
        /// Handles RoomMessageReceived event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void _room_MessagesReceived(object sender, RoomMessagesEventArgs e)
        {
            //TODO: 4.2.4 - Process messages
            _room.BeginRetrieveLatestMessages(1000, GetMessagesCallback, "Latest");

            var message = e.Messages[0];

            Messages[0].ChatMessages.Insert(0, new PersistentChatMessage()
            {
                Message = message.MessageDictionary[RoomMessageFormat.PlainText].ToString().Replace("\n", ""),
                Date = message.SentTime,
                Sender = LyncClient.GetClient().ContactManager.GetContactByUri(message.SenderUri)
                            .GetContactInformation(ContactInformationType.DisplayName).ToString().ToUpper()
            });

        }

        /// <summary>
        /// Invoked by platform thread when the message retrieve operation is completed.
        /// </summary>
        /// <param name="ar"></param>
        private void GetMessagesCallback(IAsyncResult ar)
        {

            //Declare a list to contain the collection of room messages returned by the operation
            IList<RoomMessage> messages = null;
            try
            {

                //Call the EndRetrieveMessages operation that corresponds to the
                //BeginRetrieveMessges method call that started the operation
                if (ar.AsyncState.ToString() == "Latest")
                {
                    //TODO: 4.2.5 - Call the end retrieve messages function
                    //Get the collection of the latest messages sent
                    messages = _room.EndRetrieveLatestMessages(ar);

                    if (messages.Count > 0)
                    {
                        //Invoke a delegate on the UI thread that loads the message collection in the message list box.                
                        Dispatcher.BeginInvoke(
                            (Action)delegate()
                            {
                                LoadPostedMessages(messages);
                            });
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Exception on EndRetrieveLatestMessages " + ex.Message);
            }
        }

        /// <summary>
        /// Loads a list box with a collection of messages
        /// </summary>
        /// <param name="messages"></param>
        private void LoadPostedMessages(IList<RoomMessage> messages)
        {
            string msgTmp = string.Empty;
            string lastDate = new DateTime(1000, 1, 1).Date.ToString("D").ToUpper();
            _chatRoomData = new List<PersistentChatDay>();

            var messagesByDate = messages.OrderByDescending(p => p.SentTime.Date).GetEnumerator();

            while (messagesByDate.MoveNext())
            {
                var currentDate = messagesByDate.Current.SentTime.ToString("D").ToUpper();

                msgTmp = messagesByDate.Current.MessageDictionary[RoomMessageFormat.PlainText].ToString().Replace("\n", "");

                //Add new parent element "day"
                if (currentDate != lastDate)
                {
                    var day = new PersistentChatDay() { Date = currentDate };
                    day.ChatMessages.Add(new PersistentChatMessage()
                    {
                        Date = messagesByDate.Current.SentTime,
                        Sender = LyncClient.GetClient().ContactManager.GetContactByUri(messagesByDate.Current.SenderUri)
                            .GetContactInformation(ContactInformationType.DisplayName).ToString().ToUpper(),
                        Message = msgTmp
                    });

                    //Add new instance to date
                    _chatRoomData.Add(day);
                }
                //add to existing day
                else
                {
                    //Get messages for date
                    var results = _chatRoomData.Where(p => p.Date == currentDate).FirstOrDefault();

                    results.ChatMessages.Add(new PersistentChatMessage()
                    {
                        Date = messagesByDate.Current.SentTime,
                        Sender = LyncClient.GetClient().ContactManager.GetContactByUri(messagesByDate.Current.SenderUri)
                            .GetContactInformation(ContactInformationType.DisplayName).ToString().ToUpper(),
                        Message = msgTmp
                    });
                }
                lastDate = currentDate;
            }

            foreach (PersistentChatDay day in _chatRoomData)
            {
                day.ChatMessages = new ObservableCollection<PersistentChatMessage>(day.ChatMessages.OrderByDescending(p => p.Date));
            }

            Messages = _chatRoomData;
            treeView.DataContext = Messages;
        }

        /// <summary>
        /// Represents Active Rooms message collection
        /// </summary>
        public List<PersistentChatDay> Messages { get; set; }
    }

    public class PersistentChatDay
    {
        public PersistentChatDay() { ChatMessages = new ObservableCollection<PersistentChatMessage>(); }

        public string Date { get; set; }
        public ObservableCollection<PersistentChatMessage> ChatMessages { get; set; }
    }

    public class PersistentChatMessage
    {
        public DateTime Date { get; set; }
        public string FormattedDate { get { return Date.ToShortTimeString(); } }
        public string Sender { get; set; }
        public string Message { get; set; }
    }
}
